require 'thin'

module Simplepub
  module Cli
    class Server < Thor
      desc "start", "Start simplepub server"
      option :rackup_file, desc: "Rack configuration file", default: Simplepub.rackup_file, aliases: "-R"
      option :environment, desc: "Simplepub environment", default: "development", aliases: "-E"
      option :rack_environment, desc: "Rack environment", default: "production", aliases: "-e"
      option :port, desc: "Rack port", default: "4443", aliases: "-p"
      option :address, desc: "Rack address", default: "127.0.0.1", aliases: "-a"
      option :logfile, desc: "Rack server log file", default: "/tmp/simplepub.log", aliases: "-l"
      option :pidfile, desc: "Rack server PID file", default: "/tmp/simplepub.pid", aliases: "-P"
      option :no_daemon, desc: "Don't daemonize rack server", type: :boolean, default: false
      def start
        Simplepub.config.environment = options.environment
        Simplepub.load_config

        argv = [
          "start",
          "-R", options.rackup_file,
          "-e", options.rack_environment,
          "-p", options.port,
          "-a", options.address,
          "-l", options.logfile,
          "-P", options.pidfile
        ]
        if Simplepub.config.ssl_.certificate?
          argv << "--ssl"
          # argv << "--ssl-verify"
          argv << "--ssl-cert-file"
          argv << Simplepub.config.ssl.certificate
          if Simplepub.config.ssl.keyfile?
            argv << "--ssl-key-file"
            argv << Simplepub.config.ssl.keyfile
          end
        end
        argv << "-d" unless options.no_daemon?
        say "Loading server with #{argv.inspect}", :yellow
        Thin::Runner.new(argv.flatten).run!
      end

      desc "stop", "Start simplepub server"
      option :rackup_file, desc: "Rack configuration file", default: Simplepub.rackup_file, aliases: "-R"
      option :rack_environment, desc: "Rack environment", default: "production", aliases: "-e"
      option :port, desc: "Rack port", default: "9292", aliases: "-p"
      option :address, desc: "Rack address", default: "127.0.0.1", aliases: "-a"
      option :logfile, desc: "Rack server log file", default: "/tmp/simplepub.log", aliases: "-l"
      option :pidfile, desc: "Rack server PID file", default: "/tmp/simplepub.pid", aliases: "-P"
      def stop
        Simplepub.load_config
        argv = [
          "stop",
          "-R", options.rackup_file,
          "-e", options.rack_environment,
          "-p", options.port,
          "-a", options.address,
          "-l", options.logfile,
          "-P", options.pidfile
        ]
        Thin::Runner.new(argv.flatten).run!
      end
    end
  end
end
