require 'yaml'
require 'hashie'

module Simplepub
  module Configuration
    attr_reader :config

    def configure(&block)
      yield(config)
    end

    # Resets the configuration to the default (empty hash)
    def reset_config
      @config = Hashie::Mash.new
    end

    def default_config_file
      return config.config_file if config.config_file?
      config_file = config_file_paths.detect { |path| File.exists?(path) }
      config_file
    end

    def default_environment
      config.environment? ? config.environment : "development"
    end

    # Loads the  configuration from a given YAML file and environment (such as production)
    def load_config(filename=default_config_file, environment=default_environment)
      raise ConfigurationFileError, "Configuration file not found. Available paths: #{config_file_paths.inspect}" if filename.nil?

      yaml = YAML.load_file(filename)[environment.to_s]

      raise ArgumentError, "The #{environment} environment does not exist in #{filename}" if yaml.nil?

      config.deep_merge!(yaml)
    end

    private

    def config_file_paths
      [
        "#{ENV['HOME']}/.simplepub.yml",
        "/etc/simplepub/config.yml",
        "/etc/simplepub.yml"
      ]
    end
  end
end
